<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'gipfl\\ZfDb\\' => array($vendorDir . '/gipfl/zfdb/src'),
    'gipfl\\ZfDbStore\\' => array($vendorDir . '/gipfl/zfdbstore/src'),
    'gipfl\\Web\\' => array($vendorDir . '/gipfl/web/src'),
    'gipfl\\Translation\\' => array($vendorDir . '/gipfl/translation/src'),
    'gipfl\\SystemD\\' => array($vendorDir . '/gipfl/systemd/src'),
    'gipfl\\Stream\\' => array($vendorDir . '/gipfl/stream/src'),
    'gipfl\\Socket\\' => array($vendorDir . '/gipfl/socket/src'),
    'gipfl\\SimpleDaemon\\' => array($vendorDir . '/gipfl/simple-daemon/src'),
    'gipfl\\ReactUtils\\' => array($vendorDir . '/gipfl/react-utils/src'),
    'gipfl\\Protocol\\NetString\\' => array($vendorDir . '/gipfl/protocol-netstring/src'),
    'gipfl\\Protocol\\JsonRpc\\' => array($vendorDir . '/gipfl/protocol-jsonrpc/src'),
    'gipfl\\Protocol\\Generic\\' => array($vendorDir . '/gipfl/protocol/src/Generic'),
    'gipfl\\Protocol\\Exception\\' => array($vendorDir . '/gipfl/protocol/src/Exception'),
    'gipfl\\Process\\' => array($vendorDir . '/gipfl/process/src'),
    'gipfl\\OpenRpc\\' => array($vendorDir . '/gipfl/openrpc/src'),
    'gipfl\\Log\\' => array($vendorDir . '/gipfl/log/src'),
    'gipfl\\LinuxHealth\\' => array($vendorDir . '/gipfl/linux-health/src'),
    'gipfl\\Json\\' => array($vendorDir . '/gipfl/json/src'),
    'gipfl\\InfluxDb\\' => array($vendorDir . '/gipfl/influxdb/src'),
    'gipfl\\IcingaWeb2\\' => array($vendorDir . '/gipfl/icingaweb2/src'),
    'gipfl\\IcingaCliDaemon\\' => array($vendorDir . '/gipfl/icinga-cli-daemon/src'),
    'gipfl\\Format\\' => array($vendorDir . '/gipfl/format/src'),
    'gipfl\\Diff\\' => array($vendorDir . '/gipfl/diff/src'),
    'gipfl\\DbMigration\\' => array($vendorDir . '/gipfl/db-migration/src'),
    'gipfl\\DataType\\' => array($vendorDir . '/gipfl/data-type/src'),
    'gipfl\\Curl\\' => array($vendorDir . '/gipfl/curl/src'),
    'gipfl\\Cli\\' => array($vendorDir . '/gipfl/cli/src'),
    'gipfl\\Calendar\\' => array($vendorDir . '/gipfl/calendar/src'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/Psr/Log'),
);
